from codeina.fxml import XMLObject
from codeina.extern.log import log

class ProductFlavor(XMLObject):
    """
    Describes a flavor of a product.
    """
    system = None
    arch = None
    distro = None
    distro_version = None
    core_req = None
    base_req = None
    version = None
    uri = None
    comment = None
    sha1sum = None

    def is_compatible(self, system, arch, distro, core_version,
                      base_version):
        if system and system != self.system:
            return False
        if arch and arch != self.arch:
            if self.arch == 'i686':
                if not arch == 'i586' and not arch == 'i386':
                    return False
            elif self.arch == 'i586':
                if not arch == 'i386':
                    return False
            else:
                return False
        # if we have a list it is like [distro, distro_version]
        # else we don't check for the distro version...
        if isinstance(distro, list):
            if not self.is_compatible_distro (distro):
                return False
        elif distro and distro != self.distro and self.distro != 'generic':
            return False
        if core_version and not self.core_req.fulfils_requirement(core_version):
            return False
        if base_version and not self.base_req.fulfils_requirement(base_version):
            return False
        return True

    def get_distro (self):
        return self.distro

    def get_distro_version (self):
        return self.distro_version

    def is_distro (self, distro):
        return self.distro == distro

    def is_compatible_distro (self, distro):
        if self.get_distro () == 'generic':
            # if we are generic all distros are compatible
            return True
        # distro must be the same
        if self.distro != distro[0]:
            return False
        if self.get_distro_version () == 'any':
            return True
        return self.get_distro_version () == distro[1]

    def __repr__(self):
        return ", ".join([str(i) for i in (self.system, self.arch, self.distro, self.distro_version, self.uri)])
