# -*- Mode: Python -*-
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4

from codeina.extern.log import log

_single_providers = None
def CodeinaProvider ():
    global _single_providers
    if _single_providers:
        return _single_providers
    _single_providers = CodeinaProviders ()
    return _single_providers

def has_fluendo_provider (providers_list):
    """
    Return True if the providers_list has a fluendo provider
    """
    if not isinstance (providers_list, list):
        log.warning ('providers', 'we expected a list with providers')
        return False
    for p1 in providers_list:
        if p1.id == 'fluendo':
            return True
    return False

def set_fluendo_provider_first (providers_list):
    new_providers = []
    for p1 in providers_list:
        if p1.id == 'fluendo':
            new_providers = [p1] + new_providers
        else:
            new_providers.append (p1)
    return new_providers

class CodeinaProviders:

    _xml_files = None
    _product_lists = None
    _update_urls = None

    def __init__ (self):
        self.update_xml_files ()

    def product_list_append (self, list):
        self._product_lists.append (list)

    def product_list_add_in_front (self, list):
        self._product_lists = [list] + self._product_lists

    def get_all_xml_files (self):
        def basename_in_list (f, l):
            basename = os.path.basename (f)
            for i in l:
                if os.path.basename (i) == basename:
                    return True
            return False
        if self._xml_files:
            return self._xml_files
        self._xml_files = []
        from glob import glob
        import os.path
        from codeina import configure
        log.debug ('providers', 'Updating providers files')
        for path in ( \
            os.path.join(configure.USR_PROVIDERS, '*.xml'), \
            os.path.join(configure.SYS_PROVIDERS, '*.xml'), \
            os.path.join( os.path.dirname( os.path.realpath(__file__)), '..', 'providers', '*.xml') ):
            for xml_file in glob(path):
                if not basename_in_list (xml_file, self._xml_files):
                    log.log ('providers', 'new basefile accepted: %s' % xml_file)
                    self._xml_files.append (xml_file)
                else:
                    log.debug ('providers', 'basefile already found: %s' % xml_file)
        return self._xml_files

    def get_all_xml_files_for_current_system (self):
        from codeina import check
        distro = check.get_distro_name ()
        distro_version = check.get_distro_version ()
        fluendo_name = 'fluendo.xml'
        distro_name = '%s_%s.xml' % (distro, distro_version)
        log.info ('providers', 'Only looking for files %s and %s' % (fluendo_name, distro_name))
        xml_files = []
        for f in self.get_all_xml_files ():
            if fluendo_name in f or distro_name in f:
                log.log ('providers', 'accepting file: %s', f)
                xml_files.append (f)
            else:
                log.log ('providers', 'refusing file: %s', f)
        return xml_files

    def update_xml_files (self):
        from codeina import httpdownload
        from codeina import configure
        for url in self.get_all_update_urls ():
            filename = url.split('/')[-1]
            d = httpdownload.HttpDownload()
            try:
                e = d.run(url, configure.USR_PROVIDERS, \
                        filename, None)
            except Exception, e:
                log.log ('codeina', e)
            else:
                self._xml_files = None
                self._product_lists = None

    def get_all_product_lists (self):
        if self._product_lists:
            return self._product_lists
        self._product_lists = []
        from codeina import productlistparser
        import os, sys
        import traceback
        parser = productlistparser.ProductListParser()
        self._update_urls = []
        try:
            for xml_file in self.get_all_xml_files ():
                try:
                    xml_data = open(xml_file)
                except IOError:
                    log.info('providers',
                        'parse_available: could not open %s' % xml_file)
                    continue
                try:
                    product_list = parser.parse (xml_data)
                except Exception, e:
                    log.info('providers',
                        'parse_available: could not parse %s, removing' % xml_file)
                    try:
                        os.remove(xml_file)
                    except:
                        log.info('providers',
                            'could not delete invalid xml file %s' % xml_file)
                    continue
                if not product_list.url:
                    # Add a default url
                    from codeina import configure
                    product_list.url = configure.CODEINA_PROVIDERS_PREFIX + os.path.basename (xml_file)
                    log.debug ('providers', 'setting default update url to %s' % product_list.url)
                else:
                    log.log ('providers', 'update url is: %s' % product_list.url)
                if product_list.filter_products_for_current_system ():
                    # Add the Fluendo provider in the front
                    if product_list.id == 'fluendo':
                        self.product_list_add_in_front (product_list)
                    else:
                        self.product_list_append (product_list)
                else:
                    log.debug ('providers', 'provider (%s) has no products for this system' % product_list.id)
                if not product_list.url in self._update_urls:
                    log.log ('providers', 'adding update-url: %s' % product_list.url)
                    self._update_urls.append (product_list.url)
                else:
                    log.debug('providers', '%s already in update url list' % product_list.url)
        except:
            log.warning ('providers', "Oops, couldn't load products XML list: %s" % sys.exc_info()[0])
            traceback.print_exc()
        return self._product_lists

    def get_all_update_urls (self):
        self.get_all_product_lists ()
        return self._update_urls

    def get_providers_with_id (self, id):
        product_list = []
        for product in self.get_all_product_lists():
            if product.id == id:
                product_list.append (product)
        return product_list

    def get_products_with_updates (self, id=None):
        if id:
            provider_list = self.get_providers_with_id (id)
        else:
            provider_list = self.get_all_product_lists ()
        product_with_update = []
        for provider in provider_list:
            for product in provider.products:
                if product.is_fully_installed () and product.updates_available ():
                    product_with_update.append (product)
        return product_with_update

    def get_products_installable (self, id=None):
        if id:
            provider_list = self.get_providers_with_id (id)
        else:
            provider_list = self.get_all_product_lists ()
        installable_products = []
        for provider in provider_list:
            for product in provider.products:
                if not product.is_fully_installed () or product.updates_available ():
                    installable_products.append (product)
        return installable_products

    def get_provider_id_for_product (self, product):
        for p2 in self.get_all_product_lists ():
            if product in p2.products:
                return p2.id
        return None

    def get_products_for_pattern (self, pattern):
        # build the list of required plugins from pattern
        from codeina.requiredplugin import RequiredPlugin
        log.info('codeina', "parsing pattern %s" % pattern)
        required_plugins = []
        match = RequiredPlugin(pattern)
        if match.valid:
            required_plugins.append(match)
            log.debug('codeina', 'asked to install plugin: %r return %s' % \
                    (pattern[15:], match))
        # get the list of installable plugins matching required ones
        product_list = CodeinaProvider ().get_products_installable ()
        matching_products = []
        for req in required_plugins:
            # check if this req can be converted to valid GstCaps
            import gst
            wanted_caps = gst.caps_from_string (req.get_req_string ())
            if not wanted_caps:
                log.warning ('codeina', "Failed to convert string '%s' to GstCaps" % req.get_req_string )
                continue
            for p in product_list:
                if p.provides_requirement(req):
                    p.provider = CodeinaProvider ().get_provider_id_for_product (p)
                    matching_products.append(p)
        # build the list of providers
        matching_providers, providers_rank = [], []
        for p in matching_products:
            if p.provider not in matching_providers:
                matching_providers.append(p.provider)
                # capitalize provider name 
                providers_rank.append([p.provider[0].upper() \
                    + p.provider[1:], p.rank])
        return providers_rank, matching_products

    def get_providers_for_pattern (self, pattern):
        log.log ('codeina', 'filtering for pattern: %s' % pattern)
        NotUsed, required_products = self.get_products_for_pattern (pattern)
        p1 = []
        for each_list in self._product_lists:
            p2 = each_list.get_intersection ( required_products)
            if p2.product_length () > 0:
                p1.append (p2)
        return p1
