# -*- Mode: Python -*-
# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
#
# Codeina, Codec Installation Application
# Copyright (C) 2007 Fluendo S.A.
#
# Written by Tim-Philipp Müller <tim centricular net>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

import gtk
import gobject
from codeina.httpdownload import HttpDownload, DEFAULT_DOWNLOAD_DIR
from gettext import gettext as _

class HttpDownloadDialog(HttpDownload):
    """
    Same as HttpDownload, but shows a progress dialog.
    Note that it doesn't derive from GtkDialog.
    """
    def __init__(self, parent_window=None):
        HttpDownload.__init__(self)
        self.parent_window = parent_window
        self.create_dialog()

    def dialog_response_cb(self, *args):
        if not self.done:
            dlg, response_id = args
            if response_id in (gtk.RESPONSE_REJECT, gtk.RESPONSE_DELETE_EVENT):
                self.cancelled = True
            else:
                print "Unexpected download dialog response id", response_id

    def create_dialog(self, message=None, parent_window=None):
        self.dlg = gtk.Dialog(title=_("Downloading"),\
                              parent=parent_window,  \
                              buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT))
        alignment = gtk.Alignment(0.0, 0.5, 1.0, 1.0)
        alignment.set_padding (6, 6, 6, 6)
        self.dlg_label = gtk.Label()
        if message:
            self.set_dialog_text(message)
        alignment.add(self.dlg_label)
        self.dlg.vbox.pack_start(alignment)
        self.progressbar = gtk.ProgressBar()
        self.dlg.vbox.pack_start(self.progressbar)
        self.dlg.connect('response', self.dialog_response_cb)
        self.dlg.show_all()
        self.dlg.hide()

    def set_dialog_text(self, txt):
        self.dlg_label.set_markup('<b>' + gobject.markup_escape_text(txt) + '</b>')

    def show_dialog(self):
        self.dlg.show()

    def hide_dialog(self):
        self.dlg.hide()

    def destroy(self):
        self.dlg.destroy()

    def run(self, url, directory, fn, sha1sum, message):
        if message:
            self.set_dialog_text(message)
        self.dlg.present()
        err = HttpDownload.run(self, url, directory, fn, sha1sum, None)
        self.dlg.hide()
        return err

    """ Called from the context of the main GUI thread """
    def update_progress(self):
        if self.done and not self.err and not self.cancelled:
            self.progressbar.set_text(_("Finished"))
            self.progressbar.set_fraction(1.0)
        elif self.done and (self.err or self.cancelled):
            self.progressbar.set_text('')
        elif self.bytes_read < 0:
            self.progressbar.set_text(_("Connecting to server"))
            self.progressbar.set_fraction(0.0)
        elif self.bytes_read == 0:
            self.progressbar.set_text(_("Connected to server"))
            self.progressbar.set_fraction(0.0)
        elif self.content_length > 0:
            self.progressbar.set_text(_("Downloading (%.1f KB of %.1f KB)") % (
                self.bytes_read / 1000.0, self.content_length / 1000.0))
            percent = (float) (self.bytes_read * 1.0 / self.content_length)
            self.progressbar.set_fraction(percent)
        else:
            self.progressbar.set_text(_("Downloading"))
            self.progressbar.pulse()
        return not self.done

